import java.util.Vector;

public class KDTree {
	int depth;
	double[] point;
	KDTree left;
	KDTree right;

	KDTree(double[] point, int depth) {
		this.point = point;
		this.depth = depth;
	}

	//@OFF
	double difference(double[] a) {
		return a[depth % a.length] - point[depth % point.length];

	}

	//@ON
	boolean compare(double[] a) {
		// @OFF
		return difference(a) >= 0;
		// @ON
		// @@ throw(new Error("TODO"));
	}

	// @OFF
	static KDTree insert(KDTree tree, double[] p, int depth) {
		if (tree == null)
			return new KDTree(p, depth);

		if (tree.compare(p))
			tree.right = insert(tree.right, p, depth + 1);
		else
			tree.left = insert(tree.left, p, depth + 1);

		return tree;
	}

	// @ON
	static KDTree insert(KDTree tree, double[] p) {
		// @OFF
		return insert(tree, p, 0);
		// @ON
		// @@ throw(new Error("TODO"));
	}

	static double sqDist(double[] a, double[] b) {
		// @OFF
		double d = 0.0;
		for (int i = 0; i < a.length; i++)
			d += (a[i] - b[i]) * (a[i] - b[i]);
		return d;
		// @ON
		// @@ throw(new Error("TODO"));
	}

	static double[] closestNaive(KDTree tree, double[] a, double[] champion) {
		// @OFF
		if (tree == null)
			return champion;

		if (sqDist(a, champion) > sqDist(a, tree.point))
			champion = tree.point;
		champion = closestNaive(tree.left, a, champion);
		return closestNaive(tree.right, a, champion);
		// @ON
		// @@ throw(new Error("TODO"));
	}


	static double[] closestNaive(KDTree tree, double[] a) {
		// @OFF
		if (tree == null)
			return null;
		return closestNaive(tree, a, tree.point);
		// @ON
		// @@ throw(new Error("TODO"));
	}

	static double[] closest(KDTree tree, double[] a, double[] champion) {
		if (tree == null)
			return champion;

		// sert pour InteractiveClosest.
		InteractiveClosest.trace(tree.point, champion);
		
		// @OFF
		double c = tree.difference(a);
		KDTree first, second;
		if (c < 0) {
			first = tree.left;
			second = tree.right;
		} else {
			first = tree.right;
			second = tree.left;
		}

		champion = closest(first, a, champion);
		double d = sqDist(a, champion);
		if (d >= c * c) {
			if (d > sqDist(a, tree.point))
				champion = tree.point;
			champion = closest(second, a, champion);
		}

		return champion;
		// @ON
		// @@ throw(new Error("TODO"));
	}

	static double[] closest(KDTree tree, double[] a) {
		// @OFF
		if (tree == null)
			return null;
		return closest(tree, a, tree.point);
		// @ON
		// @@ throw(new Error("TODO"));
	}

	static int size(KDTree tree) {
		// @OFF
		if (tree == null)
			return 0;
		return 1 + size(tree.left) + size(tree.right);
		// @ON
		// @@ throw(new Error("TODO"));
	}

	static void sum(KDTree tree, double[] acc) {
		// @OFF
		if (tree == null)
			return;

		for (int i = 0; i < tree.point.length; i++)
			acc[i] += tree.point[i];

		sum(tree.left, acc);
		sum(tree.right, acc);
		// @ON
		// @@ throw(new Error("TODO"));
	}

	static double[] average(KDTree tree) {
		// @OFF
		double[] acc = new double[tree.point.length];
		int size = size(tree);
		sum(tree, acc);
		for (int i = 0; i < tree.point.length; i++) {
			acc[i] /= size;
		}
		return acc;
		// @ON
		// @@ throw(new Error("TODO"));
	}

	// @OFF
	static int palette(KDTree tree, int maxpoints, Vector<double[]> acc) {
		if (tree == null || maxpoints <= 0)
			return 0;
		if (maxpoints == 1) {
			acc.add(average(tree));
			return 1;
		}
		int mid = (int) Math.round((double) maxpoints * size(tree.left) / size(tree));
		int total = palette(tree.left, mid, acc);
		total += palette(tree.right, maxpoints - mid, acc);
		if (total < maxpoints) {
			acc.add(average(tree));
			total++;
		}
		return total;
	}

	// @ON

	static Vector<double[]> palette(KDTree tree, int maxpoints) {
		// @OFF
		// throw new Error("Methode Vector<double[]> palette(KDTree, int) a completer (Question 4.2)");
		Vector<double[]> acc = new Vector<double[]>();
		palette(tree, maxpoints, acc);
		return acc;
		// @ON
		// @@ throw(new Error("TODO"));
	}

	public String pointToString() {
		StringBuffer sb = new StringBuffer();
		sb.append("[");
		if (this.point.length > 0)
			sb.append(this.point[0]);
		for (int i = 1; i < this.point.length; i++)
			sb.append("," + this.point[i]);
		sb.append("]");
		return sb.toString();
	}

}
